/* Copyright (c) Oracle Corporation 2000.  All Rights Reserved. */

package oracle.forms.demos;

import java.util.*;
import java.net.InetAddress;
import java.text.*;

import oracle.forms.ui.*;
import oracle.forms.properties.*;
import oracle.forms.handler.*;

/**
 * <H3><FONT COLOR="003399">Oracle Forms Server 6i PJC Example</FONT></H3>
 * <P>This JavaBean fetches information from the local client system and
 * delivers it to Forms when the <code>get_custom_property</code> builtin is invoked.</P>
 * <P>The class will need to be signed
 * before being used in general purpose Forms applications since access to the System properties
 * is an activity that is defined outside of the default Java sandbox.</P>
 *
 * @version 1.0 03/10/00
 * @author 	Steve Button
 */

 public class GetClientInfo
       extends VBean
{
  /**
   * Tthe CLASSNAME for this class, used in debugging
   */
  private String CLASSNAME = getClass().getName();
  /**
   * Are we debugging?
   */
  private boolean DEBUG = false;
  /**
   * The InetAddress object used to store the localhost information
   */
  private InetAddress localhost = null;
  /**
   * The default message to return for a null localhost value
   */
  private String DEFAULTMESSAGE = "NOT AVAILABLE";
  /**
   * Stores the handler for this view class
   */
  IHandler mHandler = null;
  /**
   * Register a custom property, used to invoke the getUsername() JavaBean method
   */
  static final ID USERNAME = ID.registerProperty("USERNAME");
  /**
   * Register a custom property, used to invoke the getHostname() JavaBean method
   */
  static final ID HOSTNAME = ID.registerProperty("HOSTNAME");
  /**
   * Register a custom property, used to invoke the getIPAddress() JavaBean method
   */
  static final ID IPADDRESS = ID.registerProperty("IPADDRESS");
  /**
   * Register a custom property, used to invoke the getClientDate() JavaBean method
   */
  static final ID CLIENTDATE = ID.registerProperty("CLIENTDATE");

  /**
   * The constructor for the GetUsername JavaBean class
   */
  public void GetClientInfo()
  {
    Utils.setDebug(CLASSNAME,DEBUG);
    Utils.debug(CLASSNAME,"GetUsername()");
  }

  /**
   * Method in the IView interface that is called when the component is first
   * initialised from by the Forms Server.
   *
   * @param  a reference to the Handler object for this class
   */
  public void init(IHandler handler)
  {
    Utils.debug(CLASSNAME,"init()");
    mHandler = handler;
    findLocalHost();
  }

  /**
   * Method in the IView interface that is called when the Forms Server wishes to
   * obtain the value of a specific property from the class.
   *
   * For the 6i release, the get_custom_property method will be available which will
   * allow for a far simple usage model.  The Form developer will simply be able to call
   * get_custom_property('USERNAME') and we will return the username as a string.
   *
   * @param pid the property to be returned
   * @return the value of the property that was requested
   */
  public Object getProperty(ID pid)
  {
    Utils.debug(CLASSNAME,"getProperty()" + getPropertyName(pid));

    if(pid == USERNAME)
    {
      return (getUsername());
    }
    else if(pid == HOSTNAME)
    {
      return(getHostname());
    }
    else if(pid == IPADDRESS)
    {
      return(getIPAddress());
    }
    else if(pid == CLIENTDATE)
    {
      return(getDate());
    }
    // let the VBean superclass handle the other properties
    else
      return(super.getProperty(pid));
  }

  /**
   * Find the localhost InetAddress for the client machine in which this
   * class is currently running.  This method must have been called before
   * the other convenience functions are called since the depend on the
   * value it provides.
   */
  private void findLocalHost()
  {
    if(localhost == null)
    {
      try
      {
        Utils.debug(CLASSNAME,"obtaining LocalHost info from InetAddress");
	localhost = InetAddress.getLocalHost();
      }
      catch(Exception e)
      {
        Utils.debug(CLASSNAME,"*** error obtaining LocalHost info from InetAddress");
	e.printStackTrace();
      }
    }
  }


  /**
   * Obtains the username from the client's JVM System properties with "user.name"
   * @return the client username or the default message in case of error
   */
  private String getUsername()
  {
     return System.getProperty("user.name");
  }

  /**
   * Obtains the hostname of the local client machine
   *
   * @return the hostname or the default message in case of error
   */
  private String getHostname()
  {
    if(localhost != null)
      return localhost.getHostName();
    else
      return DEFAULTMESSAGE;
  }

  /**
   * Obtains the IP address of the local client machine
   *
   * @ return the IP Address or the default message in case of error
   */
  private String getIPAddress()
  {
    if(localhost != null)
      return localhost.getHostAddress();
    else
      return DEFAULTMESSAGE;
  }

  /**
   * A convenience function to use the new ID.getName() method exposed in the 6i
   * release but which will still allow the PJC to be used in the 6.0 release
   * @param pid the property to display the name of
   * @return the property name
   */
  public String getPropertyName(ID pid)
  {
    try
    {
      return pid.getName();
    }
    // if there is an exception, it's 6.0 running!
    catch(Exception e)
    {
    }
    return null;
  }

  /**
   * Obtain the date off of the local client machine.
   *
   * @ return the current date from the client machine
   */
  public String getDate()
  {
    DateFormat format = DateFormat.getDateInstance(DateFormat.FULL);
    format.setCalendar(Calendar.getInstance());
    return format.format(format.getCalendar().getTime());
  }

  /**
   * The test method which allows class to be run from the command line to determine
   * if it is functioning correctly.
   *
   * @param args[] the command line arguments
   */
  public static void main(String args[])
  {

    System.out.println("Running GetClientInfo()");

    GetClientInfo ci = new GetClientInfo();

    Properties props = System.getProperties();
    Enumeration e = props.propertyNames();
    while(e.hasMoreElements())
    {
      String key = (String)e.nextElement();
      System.out.println(key + " = " + props.getProperty(key));
    }                     
    // call the findLocalHost function
    ci.findLocalHost();
    System.out.println("Username : " + ci.getUsername());
    System.out.println("Hostname : " + ci.getHostname());
    System.out.println("IP       : " + ci.getIPAddress());
    System.out.println("Date     : " + ci.getDate());
    System.exit(1);
  }

}
